"
COPYRIGHT (c) 2020 LabWare

stx:libscm - a new source code management library for Smalltalk/X

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License. 

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
"
"{ Package: 'stx:libscm/mercurial' }"

"{ NameSpace: Smalltalk }"

StandaloneStartup subclass:#HGMergeToolStartup
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'SCM-Mercurial-StX-Tools'
!

!HGMergeToolStartup class methodsFor:'documentation'!

copyright
"
COPYRIGHT (c) 2020 LabWare

stx:libscm - a new source code management library for Smalltalk/X

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License. 

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
"
!

documentation
"
    documentation to be added.

    [author:]
        Jan Vrany <jan.vrany@labware.com>

    [instance variables:]

    [class variables:]

    [see also:]

"
! !

!HGMergeToolStartup class methodsFor:'startup'!

main: argv
    "Application entry point. `argv` is the array of command arguments (as Array of Strings)"

    | optparser positional output rev |

    rev := 'default'.

    optparser := CmdLineParser new.
    optparser
          on: #('-o' '--output') do:[ :o | output := o asFilename ];
          on: #('-r' '--rev') do:[ :r | rev := r ];
          on: #('-h' '--help') do:[ self usage. Smalltalk exitIfStandalone:0 ].
    [
        positional := optparser parse:argv.
    ] on: CmdLineOptionError do:[:ex |
        Stderr nextPutLine: 'ERROR: '; nextPutLine: ex description.
        Smalltalk exitIfStandalone:0.        
    ].

    positional := positional collect: [:each | each asFilename ].

    positional size == 3 ifTrue: [
        "/ Assuming first invocation variant:
        "/ ... [ -o OUTPUT ] BASE LOCAL OTHER
        | base local other |

        positional do: [:each | 
            each isRegularFile ifFalse: [
                Stderr nextPutLine: 'ERROR: No such file: ', each pathName.
                Smalltalk exitIfStandalone:2.        
            ].
            each isReadable ifFalse: [
                Stderr nextPutLine: 'ERROR: File not readable: ', each pathName.
                Smalltalk exitIfStandalone:2.        
            ].
        ].

        base := positional first.
        local := positional second.
        other := positional third.

        output notNil ifTrue: [
            (output exists and:[output isWritable not]) ifTrue: [
                Stderr nextPutLine: 'ERROR: File not writable: ', output pathName.
                Smalltalk exitIfStandalone:2.        
            ] ifFalse: [
                output directory isWritable ifFalse: [
                    Stderr nextPutLine: 'ERROR: Directory not writable: ', output directory pathName.
                    Smalltalk exitIfStandalone:2.        
                ].
            ].
        ].

        [
            (HGMergeTool new base: base local: local other: other output: output)
                ifTrue:[ Smalltalk exitIfStandalone:0 ]
                ifFalse:[ Smalltalk exitIfStandalone:3 ].
        ] on: Error do: [:ex |
            Stderr nextPutAll: 'ERROR: '; nextPutLine: ex description.
            Smalltalk exitIfStandalone:4.        
        ].
        ^ self
    ].

    positional size == 1 ifTrue: [
        "/ Assuming second invocation variant:
        "/ ... [ -r REV ] FILE

        | file |

        file := positional first.
        file isRegularFile ifFalse: [
            Stderr nextPutLine: 'ERROR: No such file: ', file pathName.
            Smalltalk exitIfStandalone:2.        
        ].
        file isReadable ifFalse: [
            Stderr nextPutLine: 'ERROR: File not readable: ', file pathName.
            Smalltalk exitIfStandalone:2.        
        ].
        file isWritable ifFalse: [
            Stderr nextPutLine: 'ERROR: File not writable: ', file pathName.
            Smalltalk exitIfStandalone:2.        
        ].

        [
            (HGMergeTool new merge: file revision: rev)
                ifTrue:[ Smalltalk exitIfStandalone:0 ]
                ifFalse:[ Smalltalk exitIfStandalone:3 ].
        ] on: Error do: [:ex |
            Stderr nextPutAll: 'ERROR: '; nextPutLine: ex description.
            Smalltalk exitIfStandalone:4.        
        ].  
        ^ self.
    ].
    Stderr nextPutLine: 'ERROR: Invalid invocation (see --help)'.
    Smalltalk exitIfStandalone:2.        

    

    "
    HGMergeToolStartup main: #('-r' 'default' 'libbasic/String.st')
    HGMergeToolStartup main: #('-r' 'default' 'libwidg2/TerminalView.st')
    HGMergeToolStartup main: #('-r' 'default' 'libwidg2/VT100TerminalView.st')
    "

    "Modified: / 27-08-2020 / 10:13:19 / Jan Vrany <jan.vrany@labware.com>"
    "Modified (comment): / 26-10-2020 / 10:29:53 / Jan Vrany <jan.vrany@labware.com>"
!

usage
    "output some command-line usage infos on stderr"
    
    Stderr nextPutLine:'usage: ',self applicationName,' [ -o OUTPUT] BASE LOCAL OTHER'.
    Stderr nextPutLine:'       ',self applicationName,' [ -r REV ] FILE'.
    Stderr nextPutLine:'  -h | --help .......... output this message'.
    Stderr nextPutLine:'  -o | --output OUTPUT . save merge to OUTPUT'.
    Stderr nextPutLine:'  -r | --rev REV ....... revision to merge FILE with'.

    "Modified: / 27-08-2020 / 09:42:00 / Jan Vrany <jan.vrany@labware.com>"
! !

!HGMergeToolStartup class methodsFor:'documentation'!

version_HG

    ^ '$Changeset: <not expanded> $'
! !

